package Smash::Utils::Plot;

use strict;
use warnings;
use POSIX;
use Math::Round;
use Smash::Utils::SVG qw(:all);
use base "Exporter";

our @EXPORT_OK = qw(draw_dot_plot draw_contig_assembly draw_fragment_recruitment draw_genome_line);
our %EXPORT_TAGS = (all => [@EXPORT_OK]);

#sub draw_y_labels {
#	my ($sequence_name, $sequence_size);
#	my %SequenceLength;
#	my %SequenceOffset;
#	my @levels = (-90, -50, -20);
#	######
#	# y axis guide
#	# use internal (absolute) coordinates in x axis, and external coordinates in y axis
#	######
#
#	######
#	# sequence label
#	######
#
#	$level = $levels[0];
#	draw_text_scaled_y($FH, $level, 0, "$sequence_name (${sequence_size}bp)", 90);
#
#	######
#	# sequence split
#	######
#
#	$level = $levels[1];
#	$sign = 1;
#	for my $seq (sort {$SequenceOffset{$a} <=> $SequenceOffset{$b}} keys %SequenceOffset) {
#		draw_line_scaled_y($FH, $level+$sign*2, $SequenceOffset{$seq}-$START_Y, $level+$sign*2, $SequenceOffset{$seq}+$SequenceLength{$seq}-$START_Y, "stroke:rgb(0,0,0);stroke-width:2px");
#		if ($SequenceLength{$seq} > 250000) {
#			my $display = $seq;
#			$display =~ s/scaffold0+//;
#			draw_text_scaled_y($FH, $level-20, $SequenceOffset{$seq}-$START_Y, $display, 90);
#		}
#		$sign *= -1;
#	}
#
#	$level = $levels[2];
#
#	######
#	# query line
#	######
#
#	draw_line_scaled_y($FH, $level, 0, $level, $YSPAN, "stroke:rgb(0,0,0);stroke-width:2px");
#
#	######
#	# ytics and ytic labels
#	######
#
#	for (my $i=$START_Y; $i <= $END_Y; $i+=$MAJOR_YTIC) {
#		draw_line_scaled_y($FH, $level-9, $i-$START_Y, $level+9, $i-$START_Y, "stroke:rgb(0,0,0);stroke-width=1px");
#		draw_text_scaled_y($FH, $level-20, $i-$START_Y, nearest(0.01, $i/1000000)."Mb", 90);
#	}
#	for (my $i=$START_Y; $i <= $END_Y; $i+=$MINOR_YTIC) {
#		draw_line_scaled_y($FH, $level-5, $i-$START_Y, $level+5, $i-$START_Y, "stroke:rgb(0,0,0);stroke-width=0.25px");
#	}
#}

sub draw_contig_assembly {
	use Smash::Core;
	use Smash::Utils::SVG qw(:all);

	my ($FH, $collection, $contig_id) = @_;
	my ($smash, $dbh, $sth);
	my ($contig_name, $contig_length);

	$smash = new Smash::Core(COLLECTION => $collection);
	$smash->init();
	$dbh = $smash->get_db_handle;

	# get contig name and length

	$sth = $dbh->prepare("SELECT external_id, length FROM contig WHERE contig_id=?");
	$sth->execute($contig_id);
	($contig_name, $contig_length) = $sth->fetchrow_array();
	$sth->finish();

	$sth = $dbh->prepare("SELECT read_id, start, end FROM contig2read WHERE contig_id=?");

	# level tracking
	my $MIN_GAP = 100;
	my $max_level=1;
	my @Used;
	my $width=2;

	# init level tracking

	map {$Used[$max_level][$_] = 0} 0..$contig_length;

	# draw the contig

	draw_line_scaled_x($FH, 0, 0, $contig_length, 0);

	# draw the reads

	$sth->execute($contig_id);
	while (my ($read, $read_start, $read_end) = $sth->fetchrow_array()) {
		my $level;
		LEVEL:for ($level=1; $level <= $max_level; $level++) {
			if (level_available(\@Used, $level, max($read_start+1-$MIN_GAP, 0), min($read_end+1+$MIN_GAP, $contig_length))) {
				last LEVEL;
			}
		}

		# make new level if necessary

		if ($level > $max_level) {
			$max_level = $level;
			map {$Used[$level][$_] = 0} 0..$contig_length;
		}

		# place the read in $level

		draw_line_scaled_x($FH, $read_start+1, $level*$width, $read_end+1, $level*$width);
		map {$Used[$level][$_] = 1} ($read_start+1)..($read_end+1);
	}
	$sth->finish();
	$smash->finish();
}

sub level_available {
	my ($UsedRef, $level, $begin, $end) = @_;
	for (my $i=$begin; $i<=$end; $i++) {
		if ($UsedRef->[$level][$i] == 1) {
			return 0;
		}
	}
	return 1;
}

sub draw_genome_line {
	my ($FH, $y, $genome_label, $genome_size, $start, $end, $major_tic, $minor_tic) = @_;
	# genome label

	my $level;
	my $font_size;
	my ($DISPLAY_LABEL, $DISPLAY_TICS) = (1,1);

	if ($DISPLAY_LABEL == 1) {
		$level = $y - 25;
		$font_size = 15;
		draw_text_scaled_x($FH, 0, $level, "$genome_label (${genome_size}bp)", (font_size => $font_size));
	}

	# genome line and xtics

	$level = $y;
	$font_size = 10;
	draw_line_scaled_x($FH, 1, $level, $end-$start+1, $level, "stroke:rgb(0,0,0);stroke-width:2px");

	if ($DISPLAY_TICS == 1) {
		for (my $i=$start; $i <= $end; $i+=$major_tic) {
			draw_line_scaled_x($FH, $i-$start+1, $level-9, $i-$start+1, $level+9, "stroke:rgb(0,0,0);stroke-width=1px");
			draw_text_scaled_x($FH, $i-$start+1, $level-10, nearest(0.01, $i/1000000)."Mb", (font_size => $font_size));
		}
		for (my $i=$start; $i <= $end; $i+=$minor_tic) {
			draw_line_scaled_x($FH, $i-$start+1, $level-5, $i-$start+1, $level+5, "stroke:rgb(0,0,0);stroke-width=0.25px");
		}
	}
}

sub draw_fragment_recruitment {
	my %params = @_;

	my $FH            = $params{OUT_FILE_HANDLE};
	my $BLAST_FILE    = $params{BLAST_FILE};
	my $DISPLAY_LABEL = $params{DISPLAY_LABEL};
	my $DISPLAY_TICS  = $params{DISPLAY_TICS};
	my $X_SCALE_DOWN  = $params{X_SCALE_DOWN};
	my $Y_SCALE_DOWN  = $params{Y_SCALE_DOWN};
	my $GENOME_SIZE   = $params{GENOME_SIZE};
	my $GENOME_NAME   = $params{GENOME_NAME};
	my $GENOME_LABEL  = $params{GENOME_LABEL};
	my $COLORS        = $params{COLORS};

	my $START_POS     = $params{START_X} || 0;
	my $END_POS       = $params{END_X}   || $GENOME_SIZE;
	my $XSPAN         = $END_POS-$START_POS+1;
	my $MAJOR_XTIC    = 10**ceil(log($XSPAN-1)/log(10));
	my $MINOR_XTIC    = $MAJOR_XTIC/10;


	# external units - usually percent identity

	my $START_Y       = 100;
	my $END_Y         =  70;
	my $YSPAN         = abs($END_Y-$START_Y);
	my $MAJOR_YTIC    =  10;
	my $MINOR_YTIC    =  $MAJOR_YTIC/2;

	# The graph goes from (0,0) to ($XSPAN, $YSPAN) since YSPAN = [$START_Y, $END_Y]

	$POINT_PRECISION     = 0.00001;
	$GLOBAL_X_SCALE_DOWN = $X_SCALE_DOWN;
	$GLOBAL_Y_SCALE_DOWN = $Y_SCALE_DOWN;

	draw_rect_scaled($FH, 0, -1, $XSPAN, $YSPAN+2, "stroke:black;stroke-width:1px;fill:white;");

	# Colors

	# Make colors evenly distributed from (0, 0, 0) to (255, 255, 255)
	#my $sample_prefix = "MC20.MG";
	#my @samples = map {"$sample_prefix$_"} (1..33);
	#my $BASE    = ceil((@samples)**(1/3));
	#my %Colors  = map {my $idx = $_; $samples[$idx] => join(",", map {nearest(1, 256/($BASE-0.5)*(int($idx/$BASE**$_)%$BASE))} reverse 0..2) } 0..$#samples; 

	my %Colors  = %$COLORS;
	my $percent_precision = 20;                    # pi values will be rounded with this precision
	my $line_stroke_width = 1;
	my $separate_layers = 1;

	my $level;
	my $font_size;

	# use internal (absolute) coordinates in y axis, and external coordinates in x axis

	draw_genome_line($FH, -15, $GENOME_LABEL, $GENOME_SIZE, $START_POS, $END_POS, $MAJOR_XTIC, $MINOR_XTIC);

	# ytics
	# mandatory, since this is important
	# use internal (absolute) coordinates in x axis, and external coordinates in y axis

	$level = -15;
	$font_size = 10;
	for (my $i=$START_Y; $i >= $END_Y; $i-=$MAJOR_YTIC) {
		my $ylevel = nearest($percent_precision, ($START_Y-$i));
		draw_rect_scaled_y($FH, $level, $ylevel, abs($level), 10*$percent_precision, "fill:black;");
		draw_text_scaled_y($FH, $level-30, $ylevel+0.75*$percent_precision, "$i%", (font_size => $font_size));
	}
	$level = -7;
	for (my $i=$START_Y; $i >= $END_Y; $i-=$MINOR_YTIC) {
		my $ylevel = nearest($percent_precision, ($START_Y-$i));
		draw_rect_scaled_y($FH, $level, $ylevel, abs($level), 10*$percent_precision, "fill:black;");
	}

	# draw reads

	print $FH "<g id=\"fragments\">\n";
	open(BLAST, "<$BLAST_FILE") || die "Cannot open $BLAST_FILE: $!";
	QRY:while (<BLAST>) {
		chomp();
		my @w = split(/\t/);
		my ($query, $sbjct, $bits, $similarity, $qStart, $qEnd, $sStart, $sEnd) = ($w[0], $w[1], $w[4], $w[11], $w[17], $w[18], $w[20], $w[21]);
		if ($sbjct ne $GENOME_NAME) {
			next QRY;
		}
		if ($similarity < $params{SIMILARITY} || $bits < $params{BITS}) {
			next QRY;
		}
		if ($sEnd - $sStart < $params{MINALIGNSLEN}) {
			next QRY;
		}
		if ($sEnd < $START_POS || $sStart > $END_POS) {
			next QRY;
		}
		my $sample = join('.', (split(/\./, $query))[0,1]);
		my $color  = $Colors{$sample} || "0,0,0";
		my $ylevel = nearest($percent_precision, ($START_Y-$similarity));
		if ($separate_layers == 1) {
			my %ReverseMap = map {$Colors{$_}=>$_} keys %Colors;
			my @colors = sort {$a cmp $b} keys %ReverseMap;
			my ($index) = grep {$color eq $colors[$_]} 0..$#colors;
			$ylevel += ($line_stroke_width*2*$index);
		}
		draw_line_scaled($FH, $sStart-$START_POS+1, $ylevel, $sEnd-$START_POS+1, $ylevel, "stroke:$color;stroke-width:${line_stroke_width}px");
	}
	close(BLAST);
	print $FH "</g>\n";
}

sub min {
	my ($a, $b) = @_;
	if ($a < $b) {
		return $a;
	} else {
		return $b;
	}
}

sub max {
	my ($a, $b) = @_;
	if ($a > $b) {
		return $a;
	} else {
		return $b;
	}
}

1;
